{% extends "base.html" %}
{% block title %}Spotcrates{% endblock %}
{% block content %}
    <h2>Hi {{  spotify.me()["display_name"] }}, <small><a href="{{ url_for('sign_out') }}">[sign out]</a></small></h2>
    <a href="{{ url_for('playlists') }}">my playlists</a>
    <a href="{{ url_for('currently_playing') }}">currently playing</a>
    <a href="{{ url_for('current_user') }}">me</a>
{% endblock %}